@@ -19,13 +19,15 @@
 #ifndef __HASH_FUNCTION_HPP__
 #define __HASH_FUNCTION_HPP__
 
-#include <boost/detail/endian.hpp>
+#include <boost/predef/other/endian.h>
+#include <boost/endian/conversion.hpp>
 #include "digestlocal.h"
-#include "bswap_32.h"
 
 class HashFunction {
 
 public:
+  
+  virtual ~HashFunction() { }
 
   virtual uint32_t operator()(const char* buf, int size, bool is_interaction = false) = 0;
 
@@ -34,6 +36,8 @@ public:
 class NullHashFunction : public HashFunction {
   
   public:
+    
+  virtual ~NullHashFunction() { }
   
   virtual uint32_t operator()(const char* buf, int size, bool is_interaction = false) {
     return 1;
@@ -48,6 +52,8 @@ class MurmurHash3HashFunction : public HashFunction {
 public :
 
   MurmurHash3HashFunction(uint32_t _seed) : seed(_seed) { }
+  
+  virtual ~MurmurHash3HashFunction() { }
 
   virtual uint32_t operator()(const char* buf, int size, bool is_interaction = false) {
     return ::PMurHash32(seed, buf, size);
@@ -66,22 +72,26 @@ public:
   : HashFunction(), seed(_seed), e(_e)
   { }
   
+  virtual ~MurmurHash3LogHashFunction() { }
+  
   virtual uint32_t operator()(const char* buf, int size, bool is_interaction = false) {
     uint32_t retval = PMurHash32(seed, buf, size);
     if (is_interaction) {
       const uint32_t* src = reinterpret_cast<const uint32_t*>(buf);
-      #ifdef BOOST_BIG_ENDIAN
-      if (inverse_mapping.find(bswap_32(src[0])) == inverse_mapping.end()) throw std::logic_error("interaction is hashed before main effect!");
-      if (inverse_mapping.find(bswap_32(src[1])) == inverse_mapping.end()) throw std::logic_error("interaction is hashed before main effect!");
-      std::string key(inverse_mapping[bswap_32(src[0])]);
+      #if BOOST_ENDIAN_BIG_BYTE && !BOOST_ENDIAN_LITTLE_BYTE
+      if (inverse_mapping.find(boost::endian::endian_reverse(src[0])) == inverse_mapping.end()) throw std::logic_error("interaction is hashed before main effect!");
+      if (inverse_mapping.find(boost::endian::endian_reverse(src[1])) == inverse_mapping.end()) throw std::logic_error("interaction is hashed before main effect!");
+      std::string key(inverse_mapping[boost::endian::endian_reverse(src[0])]);
       key.append(":");
-      key.append(inverse_mapping[bswap_32(src[1])]);
-      #else
+      key.append(inverse_mapping[boost::endian::endian_reverse(src[1])]);
+      #elif !BOOST_ENDIAN_BIG_BYTE && BOOST_ENDIAN_LITTLE_BYTE
       if (inverse_mapping.find(src[0]) == inverse_mapping.end()) throw std::logic_error("interaction is hashed before main effect!");
       if (inverse_mapping.find(src[1]) == inverse_mapping.end()) throw std::logic_error("interaction is hashed before main effect!");
       std::string key(inverse_mapping[src[0]]);
       key.append(":");
       key.append(inverse_mapping[src[1]]);
+      #else
+      #error Unknown endianness
       #endif
       e[key.c_str()] = Rcpp::wrap((int) retval);
       inverse_mapping[retval] = key;
